package truebar.example;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectReader;
import com.fasterxml.jackson.databind.ObjectWriter;
import truebar.client.TruebarClient;
import truebar.client.model.rest.configuration.Configuration;
import truebar.client.model.ws.MsgInTranscript;
import truebar.client.model.ws.MsgInTranslation;

import javax.sound.sampled.*;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Scanner;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

/* *****************************************************
    !IMPORTANT!
    This file is just for demonstration purposes
    truebar.client.TruebarClient can be used asynchronously without blocking main thread instead of waiting for responses like it is done in this example.
    AudioRecording can also be performed in separate thread to minimize voice distortions (such as : skipped samples, jitter, etc).
 *******************************************************/

public class Main {

    // Static variables
    private static final ObjectReader objectReader;
    private static final ObjectWriter objectWriter;

    static {
        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        objectMapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);

        objectReader = objectMapper.reader();
        objectWriter = objectMapper.writer().withDefaultPrettyPrinter();
    }

    public static void main(String[] args) throws ExecutionException, InterruptedException, TimeoutException {

        System.out.println("Initializing microphone");
        AudioFormat format = new AudioFormat(16000.0f, 16, 1, true, false);
        DataLine.Info info = new DataLine.Info(TargetDataLine.class, format);

        if (!AudioSystem.isLineSupported(info)) {
            throw new RuntimeException("Audio format not supported.");
        }

        TargetDataLine line;
        try {
            line = AudioSystem.getTargetDataLine(format);
            line.open();
        } catch (LineUnavailableException e) {
            throw new RuntimeException("Error initializing microphone.", e);
        }

        Scanner scanner = new Scanner(System.in);
        System.out.print("Enter username: ");
        String userName = scanner.nextLine();

        System.out.print("Enter password: ");
        String password = scanner.nextLine();


        System.out.println("Creating TruebarClient...");
        try (TruebarClient truebarClient = new TruebarClient(
                "dev-keycloak.vitasis.si",
                443,
                "dev-editor-backend.true-bar.si",
                443,
                true,
                3,
                userName,
                password
        )) {
            System.out.println("Current configuration is:");
            Configuration configuration = truebarClient.getConfiguration().get(3, TimeUnit.SECONDS);
            System.out.println(objectWriter.writeValueAsString(configuration));

            System.out.println("Updating configuration");
            configuration.getStt().getDomain().setValue("MED");
            truebarClient.setConfiguration(configuration).get(3, TimeUnit.SECONDS);

            System.out.println("Creating session...");
            TruebarClient.TranscriptionSession transcriptionSession = truebarClient.openTranscriptionSession(
                    null,
                    (msgIn) -> {
                        if (msgIn instanceof MsgInTranscript) {
                            processTranscript((MsgInTranscript) msgIn);
                        } else if (msgIn instanceof MsgInTranslation) {
                            processTranslation((MsgInTranslation) msgIn);
                        } else {
                            throw new RuntimeException("Unknown message type");
                        }
                    },
                    Throwable::printStackTrace)
                    .get(5, TimeUnit.SECONDS);


            // Create app buffer for audio data
            byte[] data = new byte[line.getBufferSize() / 5];

            // Number of actual bytes read from system buffer to app buffer
            int numBytesRead;

            // Start capturing
            line.start();
            try {

                System.out.println("...CAPTURING AUDIO .....TO FINISH PRESS ENTER...");
                // Loop until user presses enter
                while (System.in.available() == 0) {
                    // Read the next chunk of data from microphone
                    numBytesRead = line.read(data, 0, data.length);

                    // If there was any bytes read send them to TruebarService
                    if (numBytesRead > 0) {
                        transcriptionSession.sendAudioChunk(ByteBuffer.wrap(data));
                    }
                }
            } catch (IOException e) {
                throw new RuntimeException("Error while waiting for user input");
            }

            System.out.println("FINISHING SESSION");
            transcriptionSession.finish().get(3, TimeUnit.SECONDS);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }

        System.out.println("ALL OK");
    }


    private static void processTranscript(MsgInTranscript msgInTranscript) {
        try {
            // Parse transcript content
            TranscriptContent[] transcriptContents = objectReader.readValue(msgInTranscript.getTranscript().getContent(), TranscriptContent[].class);

            // Join words to form hypothesis
            StringBuilder stringBuilder = new StringBuilder();
            for (TranscriptContent transcriptContent : transcriptContents) {
                if (transcriptContent.isSpaceBefore()) {
                    stringBuilder.append(" ");
                }
                stringBuilder.append(transcriptContent.getText());
            }

            // Print hypothesis
            System.out.println(stringBuilder);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void processTranslation(MsgInTranslation msgInTranslation) {
        System.out.println("TRANSLATION : " + msgInTranslation.getContent());
    }
}